START TRANSACTION ;

-- =========================================
-- preemptive cleaning

delete from study_groups
    where id not in (
        select distinct GroupID
        from students_groups
        union distinct
        select distinct GroupID
        from disciplines_groups
    );

delete from specializations
    where id not in (
        select study_groups.SpecializationID from study_groups
    );

-- ==========================================
-- create & init `years` table
CREATE TABLE IF NOT EXISTS `years` (
  `Num` int(11) NOT NULL,
  PRIMARY KEY (`Num`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

insert into years (Num)
    select distinct semesters.Year from semesters;

alter table semesters
    add key `Year_2` (`Year`);

ALTER TABLE semesters
    ADD CONSTRAINT semesters_ibfk_1 FOREIGN KEY (`Year`) REFERENCES `years` (`Num`);




-- ==========================================
-- extract semester specific group info
-- add table `groups_years`

CREATE TABLE IF NOT EXISTS `groups_years` (
  `GroupID` int(11) NOT NULL,
  `Year` int(11) NOT NULL,
  `SpecializationID` int(11) NOT NULL,
  `Name` varchar(50) CHARACTER SET utf8 DEFAULT NULL,
  PRIMARY KEY (`Year`, `GroupID`),
  KEY `GroupID` (`GroupID`),
  KEY `Year` (`Year`),
  KEY `SpecializationID` (`SpecializationID`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

ALTER TABLE groups_years
    ADD CONSTRAINT groups_years_ibfk_1 FOREIGN KEY (`GroupID`) REFERENCES `study_groups`(`ID`),
    ADD CONSTRAINT groups_years_ibfk_2 FOREIGN KEY (`Year`) REFERENCES `years`(`Num`),
    ADD CONSTRAINT groups_years_ibfk_3 FOREIGN KEY (`SpecializationID`) REFERENCES `specializations`(`ID`);

insert into groups_years (GroupID, Year, SpecializationID, Name)
    select study_groups.ID, years.Num, study_groups.SpecializationID, study_groups.Name
    from study_groups, years;

-- ==========================================
-- save faculty info
alter table study_groups
    add column FacultyID int(11) NOT NULL;

update study_groups
    set FacultyID = (select specializations.FacultyID from specializations
        where specializations.id = study_groups.SpecializationID);

alter table study_groups
    add key FacultyID (`FacultyID`),
    add CONSTRAINT study_groups_ibfk_3 FOREIGN KEY (`FacultyID`) REFERENCES `faculties` (`ID`);


-- ==========================================
-- clear redundant columns in groups

alter table study_groups
    drop foreign key study_groups_ibfk_1,
    drop key SpecializationID,
    drop key GradeID_2;

alter table study_groups
    drop column SpecializationID,
    drop column Name;

alter table study_groups
    add unique key `FacultyGroup`(`FacultyID`,`GradeID`,`GroupNum`);

COMMIT ;


